// Updates: 2006.01.13


import java.util.*;
import javax.microedition.rms.*;


/**
 * Inspired by the J2SE <code>Properties</code> class, this one uses 
 * <code>RecordStore</code>s to store properties.
 * 
 * @author Michel Deriaz
 */
public class PropsRS {
  String name;
  Hashtable ht;
  

  /**
   * Creates a new <code>PropsRS</code> object to access the
   * record store whose name is specified by the parameter.
   * @param name the name of the record store
   */
  public PropsRS(String name) {
    this.name = name;
    ht = new Hashtable();
    RecordStore rs = null;
    RecordEnumeration re = null;
    try {
      rs = RecordStore.openRecordStore(name, true);
      re = rs.enumerateRecords(null, null, false);
      while (re.hasNextElement()) {
        String pair = new String(re.nextRecord());
        int index = pair.indexOf('\t');
        set(pair.substring(0, index), pair.substring(index + 1));
      }
    }
    catch (RecordStoreException e) {}
    finally {
      try {
        re.destroy();
        rs.closeRecordStore();
      }
      catch (Exception e) {}
    }
  }

  
  /**
   * Gets the property that matches the given key.
   * @param key the key
   * @return the property that matches the given key, or null if the property 
   * does not exist
   */
  public String get(String key) {
    return (String)ht.get(key);
  }


  /**
   * Gets the property the matches the given key, assuming that the property 
   * represents an integer.
   * @param key the key
   * @return the property that matches the given key, or
   * <code>Integer.MIN_VALUE</code> if the property does not exist or does not
   * represent a valid integer
   */
  public int getInt(String key) {
    try {
      return Integer.parseInt(get(key));
    }
    catch (NumberFormatException e) {
      return Integer.MIN_VALUE;
    }
  }
  
  
  /**
   * Sets a new property. If a property with the same key already exists, the
   * old one is replaced by the new one. The key cannot contain the
   * <code>\t</code> character because this class uses it as a separator.
   * @param key the key
   * @param value the value of the property
   * @return <code>true</code> if the property is set successfully,
   * <code>false</code> if the key is null, the key contains the <code>\t</code>
   * character, or if the value is null
   */  
  public boolean set(String key, String value) {
    if (key == null || key.indexOf("\t") >= 0 || value == null) return false;
    ht.put(key, value);
    return true;
  }
  

  /**
   * Sets a new property, described by an integer. If a property with the
   * same key already exists, the old one is replaced by the new one. The key
   * cannot contain the <code>\t</code> character because this class uses it as
   * a separator.
   * @param key the key
   * @param value the value of the property
   * @return <code>true</code> if the property is set successfully,
   * <code>false</code> if the key is null, or if the key contains the
   * <code>\t</code> character
   */    
  public boolean set(String key, int value) {
    return set(key, "" + value);
  }

  
  /**
   * Saves the properties entered via the <code>set</code> methods in the 
   * record store.
   * @return <code>true</code> if the record store is saved successfully, 
   * <code>false</code> otherwise
   */
  public boolean save() {
    RecordStore rs = null;
    RecordEnumeration re = null;
    try {
      rs = RecordStore.openRecordStore(name, true);
      re = rs.enumerateRecords(null, null, false);
      while (re.hasNextElement()) rs.deleteRecord(re.nextRecordId());
      Enumeration keys = ht.keys();
      while (keys.hasMoreElements()) {
        String key = (String)keys.nextElement();
        String value = get(key);
        byte[] pair = (key + "\t" + value).getBytes();
        rs.addRecord(pair, 0, pair.length);
      }
      return true;
    }
    catch (RecordStoreException e) {
      return false;
    }
    finally {
      try {
        re.destroy();
        rs.closeRecordStore();
      }
      catch (Exception e) {}
    }
  }
  

  /**
   * Deletes a record store. Midlets suites are only allowed to delete their own
   * record stores.
   * @param name the name of the record store
   * @return <code>true</code> if the record store is deleted successfully,
   * <code>false</code> if the record store can not be deleted
   */  
  public static boolean delete(String name) {
    try {
      RecordStore.deleteRecordStore(name);
      return true;
    }
    catch (RecordStoreException e) {
      return false;
    }
  }
}